// Contains class definitions
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
#include "MaintainDB.h"
#include <qframe.h>
#include <string.h>


// ***************
//  Here, we define the actual functions



//||||||||||||||||||||||||||||||||||||||||||||||||
//
//  Class MaintainDB
//
//||||||||||||||||||||||||||||||||||||||||||||||||



// Constructor for the MaintainDB object.
//
MaintainDB::MaintainDB(  AudioDev *devname, char *wmcatname,
					char *tkdbpath ):
   APDialog( NULL, NULL, TRUE, STYLEFLAGS )
{
   QFont dffont( "helvetica", 10 );

   dev = devname;
   hasWMDB = hasTKDB = false;
   saveTo = dbtNone;
   readFrom = dbtNone;

   if(wmcatname != NULL)
	 {
	 hasWMDB = true;
	 saveTo = (enum DBType)(saveTo | dbtWM);
	 readFrom = (enum DBType)(readFrom | dbtWM);
	 wmcddb.setName(wmcatname);
	 wmcddb.open();
	 }

   if(tkdbpath != NULL)
	 {
	 hasTKDB = true;
	 saveTo = (enum DBType)(saveTo | dbtTK);
	 readFrom = (enum DBType)(readFrom | dbtTK);
	 tkcddb.setName(tkdbpath);
	 tkcddb.open();
	 }

   setCaption( "Edit Playlist Info" );
   createChildren(dffont);
   connectChildren();
   resize( 390, 400 );
}


// Destructor for the MaintainDB object.
//
MaintainDB::~MaintainDB( void )
{
   delete saveb;
   delete closeb;
   delete cancelb;
   delete undob;
   delete tracklist;
   delete track_eb;
   delete title_eb;
   delete artist_eb;
   delete edit_txt;
   delete tlb_txt;
   delete aeb_txt;
   delete teb_txt;
   delete ctrlb_frame;
}


// Creates children for the dialog box proper
//
void MaintainDB::createChildren(QFont& dffont)
{
   ctrlb_frame = new QFrame( this, "Frame_2" );
   ctrlb_frame->setGeometry( 310, 0, 80, 310 );
   ctrlb_frame->setFrameStyle( 49 );
   
   saveb = new QPushButton( this, "saveb" );
   saveb->setGeometry( 320, 230, 60, 30 );
   saveb->setText( "Save" );
   saveb->setAutoRepeat( FALSE );
   saveb->setAutoResize( FALSE );
   
   closeb = new QPushButton( this, "closeb" );
   closeb->setGeometry( 320, 270, 60, 30 );
   closeb->setText( "Close" );
   closeb->setAutoRepeat( FALSE );
   closeb->setAutoResize( FALSE );
   
   cancelb = new QPushButton( this, "cancelb" );
   cancelb->setGeometry( 320, 10, 60, 30 );
   cancelb->setText( "Cancel" );
   cancelb->setAutoRepeat( FALSE );
   cancelb->setAutoResize( FALSE );
   
   undob = new QPushButton( this, "undob" );
   undob->setGeometry( 350, 370, 30, 20 );
   undob->setFont( dffont );
   undob->setText( "Undo" );
   undob->setAutoRepeat( FALSE );
   undob->setAutoResize( FALSE );
   
   tracklist = new QListBox( this, "tracklist" );
   tracklist->setGeometry( 10, 100, 270, 250 );
   tracklist->setFont( dffont );
   tracklist->setFrameStyle( 50 );
   tracklist->setLineWidth( 2 );
   tracklist->setDragSelect( FALSE );

   track_eb = new QLineEdit( this, "track_eb" );
   track_eb->setGeometry( 10, 370, 330, 20 );
   track_eb->setText( "" );
   track_eb->setMaxLength( MAXCHARLEN );
   track_eb->setEchoMode( QLineEdit::Normal );
   track_eb->setFrame( TRUE );
   
   title_eb = new QLineEdit( this, "title_eb" );
   title_eb->setGeometry( 55, 40, 230, 20 );
   title_eb->setText( "" );
   title_eb->setMaxLength( MAXCHARLEN );
   title_eb->setEchoMode( QLineEdit::Normal );
   title_eb->setFrame( TRUE );
   
   artist_eb = new QLineEdit( this, "artist_eb" );
   artist_eb->setGeometry( 55, 10, 230, 20 );
   artist_eb->setText( "" );
   artist_eb->setMaxLength( MAXCHARLEN );
   artist_eb->setEchoMode( QLineEdit::Normal );
   artist_eb->setFrame( TRUE );
   
   edit_txt = new QLabel( this, "edittxt" );
   edit_txt->setGeometry( 10, 350, 250, 20 );
   edit_txt->setFont( dffont );
   edit_txt->setText( "Enter new label.  \"Undo\" restores the old label:" );
   edit_txt->setAlignment( 289 );
   edit_txt->setMargin( -1 );
   
   tlb_txt = new QLabel( this, "trackboxlabel" );
   tlb_txt->setGeometry( 10, 80, 130, 20 );
   tlb_txt->setFont( dffont );
   tlb_txt->setText( "Select Track Label to Edit:" );
   tlb_txt->setAlignment( 289 );
   tlb_txt->setMargin( -1 );
   
   aeb_txt = new QLabel( this, "aeb_txt" );
   aeb_txt->setGeometry( 10, 10, 40, 20 );
   aeb_txt->setText( "Artist:" );
   aeb_txt->setAlignment( 289 );
   aeb_txt->setMargin( -1 );
   aeb_txt->raise();
   
   teb_txt = new QLabel( this, "teb_txt" );
   teb_txt->setGeometry( 10, 40, 40, 20 );
   teb_txt->setText( "Title:" );
   teb_txt->setAlignment( 289 );
   teb_txt->setMargin( -1 );
   teb_txt->raise();
}


// Connects the children to the various slots
//
void MaintainDB::connectChildren(void)
{
   connect(closeb, SIGNAL(clicked()), SLOT(accept()) );
   connect(cancelb, SIGNAL(clicked()), SLOT(reject()) );
   connect(saveb, SIGNAL(clicked()), SLOT(doSave()) );
   connect(undob, SIGNAL(clicked()), SLOT(doUndo()) );
   connect(title_eb, SIGNAL(returnPressed()), 
		 SLOT(newTitleStr()) );
   connect(artist_eb, SIGNAL(returnPressed()), 
		 SLOT(newArtistStr()) );
   connect(track_eb, SIGNAL(returnPressed()), 
		 SLOT(newTrackStr()) );
   connect(tracklist, SIGNAL(highlighted(int)), 
		 SLOT(changeTrackEdit(int)) );
}



//--------------------------------------------------------------
//
// Slots [public and protected]
//
//--------------------------------------------------------------




// Public Slot to check the database for the current Audio device's
// catalog.
//
void MaintainDB::checkCatalog(void)
{
   bool found;
   if( ((dev->ToC) == NULL) || !(hasWMDB || hasTKDB) )
	 return;

   found = false;
   if(hasWMDB && (readFrom && dbtWM) )
	 {
	 found = wmcddb.read( (*(dev->ToC)) );
	 }

   if(hasTKDB && (readFrom && dbtTK) && !found)
	 {
	 found = tkcddb.read( (*(dev->ToC)) );
	 }

   if(!found)
	 return;

   dev->updateToCTrkInf();
#ifdef DEBUG_ON
   cerr << "%dbg%\tMaintainDB::checkCatalog(): updated track info"
	   << endl << "\t";
   int i;
   for(i=0; i<=(dev->ToC->len()); i++)
	 cerr << dev->ToC->time(i) << " " << flush;
   cerr << endl << "\t\tmagic=" << hex << dev->ToC->magic() 
	   << dec << endl;
#endif
   dev->cleanDisplay();
}


// Public Slot to open the dialog and start the local execution loop.
//
void MaintainDB::open(void)
{
   int status,i;
   char buf[MAXCHARLEN];

   // Make sure we have something to change...
   if( (dev->ToC) == NULL )
	 {
	 QMessageBox::message("Error!",
					  "Nothing to program!",
					  "close");
	 return;
	 }

   // Okay - there's something there.  Let's set up the track list
   // box.
   catcopy = (*(dev->ToC));
   tracklist->clear();
   for(i=0; i<catcopy.len(); i++)
	 {
	 sprintf(buf,"%d :: %.248s",i+1,catcopy.name(i));
	 tracklist->insertItem(buf);
	 }

   // Set some default contents and focus
   tracklist->setCurrentItem(0);
   changeTrackEdit(0);
   title_eb->setText(catcopy.title());
   title_eb->selectAll();
   artist_eb->setText(catcopy.artist());
   artist_eb->selectAll();
   artist_eb->setFocus();

   // Enter the control loop.
   status = exec();
   if(status == Accepted)
	 {
	 *(dev->ToC)  = catcopy;
	 dev->cleanDisplay();
	 }
}


// Protected Slot to undo any changes to the track names.
// 
void MaintainDB::doUndo(void)
{
   changeTrackEdit(tracklist->currentItem());
}


// Protected Slot to save the catalog
// 
void MaintainDB::doSave(void)
{
   if( !(hasWMDB || hasTKDB) )
	 return;

   if(hasWMDB && (saveTo && dbtWM) )
	 {
	 wmcddb.add(catcopy);
	 }

   if(hasTKDB && (saveTo && dbtTK) )
	 {
	 tkcddb.add(catcopy);
	 }
}


// Protected Slot to change the title
// 
void MaintainDB::newTitleStr(void)
{
   title_eb->selectAll();
   catcopy.setTitle(title_eb->text());
   track_eb->setFocus();   
}


// Protected Slot to change the artist
// 
void MaintainDB::newArtistStr(void)
{
   artist_eb->selectAll();
   catcopy.setArtist(artist_eb->text());
   title_eb->setFocus();
}


// Protected Slot to change a track name
// 
void MaintainDB::newTrackStr(void)
{
   unsigned int ci;
   int scrolllim;
   char buf[MAXCHARLEN];

   scrolllim = tracklist->count() - tracklist->numItemsVisible() + 1;
   ci = tracklist->currentItem();
   // we can make this cast because Catalog::setName() copies the
   // string.
   catcopy.setName( ci, (char *)(track_eb->text()) );
   sprintf(buf,"%d :: %.248s",ci+1,catcopy.name(ci));
   tracklist->changeItem(buf,ci);

   ci++;
   if( ci < tracklist->count() )
	 {
	 changeTrackEdit(ci);
	 tracklist->setCurrentItem(ci);
	 if ((int)ci <= scrolllim)
	    {
	    tracklist->setTopItem(ci);
	    }
	 }
   else
	 {
	 changeTrackEdit(0);
	 tracklist->setCurrentItem(0);
	 tracklist->setTopItem(0);
	 artist_eb->setFocus();
	 }
}


// Protected Slot to change the stuff in the track edit box
// 
void MaintainDB::changeTrackEdit(int nutrack)
{
   track_eb->setText(catcopy.name(nutrack));
   track_eb->selectAll();
}



//||||||||||||||||||||||||||||||||
//
// End
//
