// Contains class definitions
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
#include "pControl.h"



//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//
// Button pixmap defines.
//
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// The "minimize" or "iconify" button
//
RawPixData iconifyd={ 
  TRUE,
  1.0,
  1.0,
  16,
  1,
  TRUE,
  {".3125,.3125  .6874,.3125  .5,.6874  .49,.6874  .3125,.3125",
   "","","","","","","","",""},
  "5",
  "p" };


// The Eject button - a triangle and a bar
//
RawPixData ejectd={ 
  FALSE,
  1.0,
  1.0,
  32,
  2,
  TRUE,
  {".25,.5  .25,.49  .49,.25  .5,.25  .74,.49  .74,.5  .25,.5",
   ".25,.625  .74,.625  .74,.6875  .25,.6875  .25,.625",
   "","","","","","","",""},
  "7 5",
  "pp" };


// The Load button - a folder
//
RawPixData folderd={ 
  FALSE,
  1.0,
  1.0,
  32,
  5,
  FALSE,
  {".25,.375  .25,.74  .75,.74  .75,.375  .25,.375",
   ".25,.375 .25,.25  .25,.25 .45,.25  .45,.25 .45,.375",
   ".25,.475 .35,.375  .25,.575 .45,.375  .25,.675 .55,.375",
   ".324,.74 .65,.375  .424,.74 .75,.375  .524,.74 .75,.475",
   ".624,.74 .75,.575  .724,.74  .75,.675",
   "","","","",""},
  "5 6 6 6 4",
  "pll" };


// The Power button - a circle around a vertical bar
//
RawPixData powerd={ 
  FALSE,
  1.0,
  1.0,
  32,
  2,
  FALSE,
  {".25,.25  .74,.74",
   ".49,.375  .49,.624  .5,.375  .5,.624",
   "","","","","","","",""},
  "2 4",
  "cl" };





// ***************
//  Here, we define the actual functions




//||||||||||||||||||||||||||||||||||||||||||||||||
//
//  Class pControl
//
//||||||||||||||||||||||||||||||||||||||||||||||||




// Constructor for the Control buttons
//
pControl::pControl( QWidget *parent, float bscale, 
				bool useEject ) : QWidget( parent )
{
   // Some setup.
   QFont deffnt;
   cbw_p = QPoint( 0, 0 );
   isLocked = false;
   hasTitlebar = false;
   col_locked = COLOR_LOCKED;
   col_unlocked = COLOR_UNLOCKED;

   // Now set up the look and feel.
   createChildren( bscale, useEject );
   changeFont( deffnt );
   moveAll( cbw_p );
   lower();
}


// Destructor for the Control buttons
//
pControl::~pControl( void )
{
   delete lockb;
   delete locklight;
   delete titleb;
   delete powerb;
   delete iconifyb;
   delete loadejectb;
}


// Creates the child widgets and connects some signals and slots.
//
void pControl::createChildren( float bscale, bool useEjectPix )
{
  // Create the lock button and the titlebar toggle. Also connects to
  // the slots.
  lockb = new QCheckBox( LABEL_UNLOCKED, this );
  connect( lockb, SIGNAL(clicked()),
		 this, SLOT(toggle_lock()) );

  locklight = new QLabel( this );
  locklight->setBackgroundColor( col_unlocked );

  titleb = new QCheckBox( LABEL_WINTITLEBAR, this );
  connect( titleb, SIGNAL(clicked()),
		 this, SLOT(toggle_titlebar()) );
  
  // Create the power, iconify and load/eject buttons.
  powerb = new PixButton( this, powerd, bscale );
  iconifyb = new PixButton( this, iconifyd );
  if(useEjectPix)
	loadejectb = new PixButton( this, ejectd, bscale );
  else
	loadejectb = new PixButton( this, folderd, bscale );
}


// Changes the palette for the entire widget
//
void pControl::changePalette(QPalette& p)
{
  setPalette(p);
  lockb->setPalette(p);
  locklight->setPalette(p);
  titleb->setPalette(p);
  powerb->setPalette(p);
  iconifyb->setPalette(p);
  loadejectb->setPalette(p);

  powerb->bRepaint();
  iconifyb->bRepaint();
  loadejectb->bRepaint();

  if( isLocked )
	locklight->setBackgroundColor( col_locked );
  else
	locklight->setBackgroundColor( col_unlocked );
}


// Changes the font, then resizes & repositions everything.
//
void pControl::changeFont(QFont& f)
{
   setFont(f);
   lockb->setFont(f);
   titleb->setFont(f);

   locateChildren();
   moveChildren();
}


// Initializes the positions of the buttons
//
void pControl::locateChildren( void )
{
  titleb->resize( titleb->sizeHint() );
  lockb->resize( lockb->sizeHint() );
  locklight->resize( powerb->width(), bsep );

  // Now move stuff around.
  pwr_p = QPoint( bsep, bsep );
  llt_p = pwr_p + QPoint( 0, bsep + powerb->height() );
  ej_p = llt_p + QPoint( 0, 4*(bsep + locklight->height()) );
  tbt_p = ej_p + QPoint( 0, loadejectb->height() + 2*bsep );
  ico_p = tbt_p + QPoint( 0, 2*bsep + titleb->height() );

  cbw_s = QSize( (pwr_p.x() + powerb->width() + bsep),
			  (ico_p.y() + iconifyb->height() + bsep)
			  );

  loc_p = ico_p + QPoint( 2*iconifyb->width() + bsep, 0 );
  lock_w = lockb->width() + (int)(1.5*bsep);

  // Lastly, resize the main window.
  resize( loc_p.x()+lock_w, cbw_s.height() );
}


// Moves all of the children en-masse.
//
void pControl::moveChildren( void )
{
  powerb->move( cbw_p + pwr_p );
  loadejectb->move( cbw_p + ej_p );
  iconifyb->move( cbw_p + ico_p );
  titleb->move( cbw_p + tbt_p );
  lockb->move( cbw_p + loc_p );
  locklight->move( cbw_p + llt_p );
}


// Reimplemented virtual function to receive a mouse pressed
// event. Sends out the pressed() signal.
//
void pControl::mousePressEvent( QMouseEvent *me )
{
   emit pressed( mapToGlobal(me->pos()) );
}


// Reimplemented virtual function to receive a mouse move event.  It
// sends out the moving() signal.
//
void pControl::mouseMoveEvent( QMouseEvent *me )
{
   emit moving( mapToGlobal(me->pos()) );
}


// Private slot for Locking the Power and Load/Eject buttons
//
void pControl::toggle_lock()
{
  isLocked = lockb->isChecked();
  if( isLocked )
	{
	lockb->setText(LABEL_LOCKED);
	locklight->setBackgroundColor( col_locked );
	}
  else
	{
	lockb->setText(LABEL_UNLOCKED);
	locklight->setBackgroundColor( col_unlocked );
	}

  emit lockThis(isLocked);
}


// Private slot for turning the window titlebar off and on.
//
void pControl::toggle_titlebar()
{
  hasTitlebar = titleb->isChecked();

  emit titlebarOn(hasTitlebar);
}





//||||||||||||||||||||||||||||||||
//
// End.
//
