//-*-C++-*- Header file for the PixButton Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _PixButton_h_
#define _PixButton_h_ 1
#include <qpushbt.h>
#include "PixData.h"



/**
  This class is a derivative of {\bf QPushButton} designed
  specifically for use with pixmaps drawn by a \Ref{PixData} object.
  The size of each {\bf PixButton} is identical to the size of the
  \Ref{PixData} object associated with it.

  I'm sure there's probably a better way to implement this, but this
  was not only my first attempt at a Qt widget, but also my first
  attempt at C++!
  
  @memo Special {\bf QPushButton} class.
  @see RawPixData,PixData
  */
class PixButton : public QPushButton
{
   public:
	 /**
	   Class constructor.  Creates a blank {\bf PixButton}.
	   @memo
	   */
	 PixButton( QWidget *parent );

	 /**
	   Overloaded class constructor.  Equivalent to calling {\tt
	   PixButton(parent)} followed by a call to {\tt
	   bDataInit(rawdat,scale)}.
	   @see bDataInit
	   @memo
	   */
	 PixButton( QWidget *parent, RawPixData rawdat, 
			  float scale=1.0 );

	 /**
	   This method rescales the \Ref{PixData} object of the {\bf
	   PixButton}.  This also changes the size of the {\bf Pixbutton}
	   itself.  It implicitly calls \Ref{bRepaint}().
	   @memo
	   */
	 void bRescale( float scale );

	 /**
	   This method associates the {\bf PixButton} with a
	   \Ref{PixData} object.

	   The first argument is a \Ref{RawPixData} structure, which {\bf
	   bDataInit}() converts to a \Ref{PixData} object, rescaling it
	   to {\it scale} at the same time.  You can define a bunch of
	   constant \Ref{RawPixData} variables in a header file, each of
	   which defines a \Ref{PixData} to use on a {\bf PixButton}.  

	   @see RawPixData, PixData
	   @memo
	   */
	 void bDataInit( RawPixData rawdat, float scale=1.0 );

	 /**
	   Reconverts the \Ref{PixData} of this {\bf PixButton} to a {\bf
	   QPixmap} and redraws the object.

	   You need to call this if you change any of the properties of a
	   {\bf PixButton} for those changes to appear.  For example, if
	   you changed the background color of a {\bf PixButton}, you'd
	   need to do a {\tt bRepaint()}.
	   @memo
	   */
	 void bRepaint(void);

   protected:
	 /**
	   Stores the {\bf QPixmap} displayed on the button.
	   @memo
	   */
	 QPixmap b_pm;

	 /**
	   Pointer to the \Ref{PixData} object that contains the image to
	   draw on the button.
	   @memo
	   */
	 PixData *b_pd;
};


#endif //_PixButton_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
