//-*-C++-*- Header file for the PixData Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _PixData_h_
#define _PixData_h_ 1
#include <qpixmap.h>

#define MAXOBJS 10


//
typedef struct rawpixdata
{
	 bool fixed;
	 float w0;
	 float h0;
	 int scale;
	 int nobj;
	 bool filled;
	 char *coordstr[MAXOBJS];
	 char *npointstr;
	 char *drawtype;
} RawPixData;


/**
  @name RawPixData

  \\
  \\
  {\Large struct {\bf RawPixData}}
  \begin{itemize}
     \item bool {\bf fixed}
     \item float {\bf w0}
     \item float {\bf h0}
     \item int {\bf scale}
     \item int {\bf nobj}
     \item bool {\bf filled}
     \item char *{\bf coordstr}[{\tt MAXOBJS}]
     \item char *{\bf npointstr}
     \item char *{\bf drawtype}
  \end{itemize}


  {\Large {\bf Description:}}

  {\bf RawPixData} is an aggregate {\tt struct} used to feed data to
  the \Ref{PixData} class, which converts the data to a {\bf
  QPixmap}.

  Each {\bf RawPixData} variable defines up to {\tt MAXOBJS}
  "thingies" to draw.  Each "thingy" consists of a series of
  coordinate pairs describing the vector positions to use when
  drawing.  There are three types of "thingies" you can draw using
  {\bf RawPixData}:
  \begin{itemize}
      \item circles
	 \item N-sided polygons
	 \item Sequence of line segments
  \end{itemize}
  Currently, {\tt MAXOBJS}==10.

  Because you can't assign values to 2D C-arrays without pre-defining
  the array dimensions, we store some of the data for all of the
  "thingies" as strings.  That way, you can create a {\bf QPixmap}
  using less than {\tt MAXOBJS} "thingies".

  I have since thought about creating a format for the \Ref{PixData}
  class that is nothing but an array of strings, each of which
  contains data for a \Ref{PixData} object.  However, I've since
  decided that it ain't broke, so why fix it?  If there's {\em really}
  a big need for vector-drawn pixmaps, someone can always modify this
  class and create that format.
  \\
  \\
  \\
  Please note:  The following documentation of the {\tt struct}
  members is {\em not in order}.  The list at the beginning of this
  documentation specifies that.  I've reordered the descriptions of
  the members to make the documentation flow more smoothly.
  \\
  \\
  \\
  \begin{itemize}
     \item int {\bf scale} \\
	   This is something of a conversion factor: it specifies how to
	   convert from vector units to pixels.  Together with {\bf w0}
	   and {\bf h0}, {\bf scale} specifies the size of the graphic
	   you're defining.  Specifically: \\
	   \\
	   \begin{itemize}
	     \item pixmap width = {\tt scale*w0}
	     \item pixmap height = {\tt scaleh0}
	   \end{itemize} \\
	   You can set {\tt scale=1} and set {\bf w0} and {\bf h0} to the
	   width and height in pixels of the graphic.  It is easier,
	   however, to decide on a fixed coordinate system that you can
	   easily subdivide and use {\bf scale} to convert from these
	   "normalized" units to pixels. \\
	   \\
	   See {\bf w0} and {\bf h0} for more info. \\
	   \\
     \item float {\bf w0} \\
	   The width of the graphic you're defining, in "normalized"
	   units.
     \item float {\bf h0} \\
	   The height of the graphic you're defining, in "normalized"
	   units. \\
	   \\
	   The "normalized units" are just convenient units you decide
	   upon, units that make the coordinates easier to visualize.
	   They also allow you to specify the size of the graphic [as
	   well as the size and position of the things you're drawing] in
	   terms on proportions. \\
	   \\
	   For example, several of the pixmaps used in the \Ref{AudioPanel}
	   have {\tt h0=1.0} and {\tt w0=2.0} - that is, the pixmaps are
	   twice as wide as they are tall.  This also makes it {\em
	   really} easy to remember where the center of the graphic is:
	   (1.0,0.5).  By using these "normalized" coordinates, {\bf
	   scale} then specifies the size of the graphic in pixels. \\
	   \\
     \item bool {\bf fixed} \\
	   If set, indicates that the resulting {\bf QPixmap} has a fixed
	   size and should never be rescaled by \Ref{PixData}. \\
	   \\
     \item bool {\bf filled} \\
	   If set, indicates that circles and polygons should be filled
	   with the foreground color.  If {\tt filled==false},
	   circles and polygons are not filled. \\
	   \\
     \item int {\bf nobj} \\
	   The number of "thingies" you're going to draw on this graphic
	   - i.e. circles, polygons, and lines.  It must be a number less
	   than {\tt MAXOBJS} [that means < 10 currently]. \\
	   \\
     \item char *{\bf drawtype} \\
	   An array of characters indicating the type of "thingy" to
	   draw.  {\em NOTE:  Do not treat this as a string!}  I.e. don't
	   put any whitespaces between the characters. The possible
	   non-whitespace characters are: \\
	   \begin{description}
	      \item['p'] A polygon.
	      \item['c'] A circle.
	      \item['l'] A sequence of line segments.
	   \end{description} \\
	   There should be exactly {\bf nobj} elements in this array. \\
	   \\
	   For example, if {\tt nobj==4}, then {\tt drawtype="pcp"}
	   indicates that the first object is a polygon, the second is a
	   circle, and the third is another polygon. \\
	   \\
     \item char *{\bf npointstr} \\
	   A string containing {\bf nobj} numbers, separated by
	   whitespace.  The numbers are the lengths of each "array" of
	   coordinates specified in {\bf coordstr}.  So, {\tt
	   npointstr="3 7 5 5 10"} indicates that the 3{\em rd} object
	   contains 5 points. \\
	   \\
	   See {\bf coordstr} for more info. \\
	   \\
     \item char *{\bf coordstr}[{\tt MAXOBJS}] \\
	   An array of strings containing the data of each object.  This
	   is the meaty part of the {\bf RawPixData} structure. \\
	   \\
	   The first {\bf nobj} strings in the array {\em must} contain
	   coordinates for the object being drawn.  Each coordinate is a
	   piar of floating point numbers separated by a comma.  The
	   coordinates in each string are separated by normal
	   whitespace. The number of coordinates in {\tt coordstr[i]}
	   {\em MUST} be the same as the i{\em th} value specified in
	   {\bf npointstr}. \\
	   \\
	   There are some additional restrictions placed on {\bf
	   coordstr}: \\
	   \\
	   \begin{itemize}
	      \item {\tt drawtype[i]=='c'} \\
		 {\tt coordstr[i]} should contain exactly two coordinates:
		 the upper-left and lower-right corners of a bounding box
		 for a circle.
		 \item {\tt drawtype[i]=='p'} \\
		 {\tt coordstr[i]} contains the corners of a polygon.  For a
		 square or rectangle, you need 5 coordinates.  For an
		 octagon, you'd need 9.
		 \item {\tt drawtype[i]=='l'} \\
		 You might expect that you can only specify two coordinates
		 in {\tt coordstr[i]}, but this isn't the case.  You must
		 have an {\em even} number of coordinates.  This way, you
		 can draw several line segments using only one "object".
		 The first coordinate is the starting point of the first
		 line segment.  The third coordinate is the starting point
		 of the second line segment.  And so on...
	   \end{itemize} \\

	   The coordinates should be positive floating point numbers and
	   should be less than {\bf w0} and {\bf h0} for the x- and
	   y-coordinate, respectively. The coordinate " 0.0,0.0 " is in the
	   upper-left corner of the pixmap. \\
	   \\
	   There is one final, but important, subtlety to the
	   coordinates.  It has to do with roundoff error. \\
	   \\
	   Suppose you had a square pixmap:  {\tt w0=1.0, h0=1.0}.  You
	   want to draw a circle in the center of this pixmap.  So, you
	   set up a {\bf RawPixData} variable and set:\\
	   {\tt coordstr=#{#"0.3,0.3  0.7,0.7",\\
	   "","","","","","","","",""#};#} \\
	   You'll get a circle, but it won't be centered.  At least, not
	   always.  The circle {\em may} be centered at some scales, but
	   a bit off-center at others.  This is due to roundoff error,
	   and it defeats the whole purpose of a vector-drawn pixmap. \\
	   \\
	   It turns out that, because of roundoff error, the coordinate
	   "0.5,0.5" isn't exactly the center.  Pixels aren't
	   infinitessimally small.  So, to take roundoff into account,
	   you need to subtract "by a little bit".  How much is enough?
	   That rather depends:  what precision of floating point number
	   do you use in your coordinates?  What's the largest size you
	   ever intend to scale this baby to?  In general, one order of
	   magnitude less than precision works well.  So, in our example,
	   we should really have:\\
	   {\tt coordstr=#{#"0.3,0.3  0.69,0.69",\\
	   "","","","","","","","",""#};#} \\
	   If, however, you intended to scale this baby up to 1000x1000
	   pixels, you might want to use:\\
	   {\tt coordstr=#{#"0.3,0.3  0.699,0.699",\\
	   "","","","","","","","",""#};#} \\
	   \\
	   As you can see, you'll need to fiddle with this a bit.
  \end{itemize}


  {\Large {\bf Example}}

  To end this documentation, I want to give you an example.  This is
  the actual, honest-to-goodness definition of the {\bf RawPixData}
  variable for the power button icon.  It's direct from the source
  code, and specifies two line segments, side-by-side, centered inside
  of a circle. \\
  \\

  {\tt
  \begin{verbatim}
  RawPixData powerd={
     FALSE,
     1.0,
     1.0,
     32,
     2,
     FALSE,
     {".25,.25  .74,.74",
     ".49,.375  .49,.624  .5,.375  .5,.624",
     "","","","","","","",""},
     "2 4",
     "cl" };
  \end{verbatim}
  }
  
  @memo Holds a \Ref{PixData} object in raw form.
  */



/**
  The {\bf PixData} object stores information used to create a
  vector-drawn {\bf QPixmap}.

  Performs rescalings and conversion to a QPixmap.  Data for the
  drawing is fed in as a \Ref{RawPixData} variable.

  @memo Vector-drawn {\bf QPixmap}s
  */
class PixData
{
   public:
	 /**
	   Holds the size of the {\bf QPixmap}.\\
	   You should not change this variable.
	   @memo
	   */
	 QSize dsize;
	 /**
	   Flag to indicate if this {\bf PixData} can't be rescaled.\\
	   You should not change this variable.
	   @see dcoords
	   @memo
	   */
	 bool fixedScale;
	 /**
	   Number of objects to draw.\\
	   You should not change this variable.
	   @see dcoords
	   @memo
	   */
	 int nobj;
	 /**
	   Flag to indicate whether circles and polygons should be
	   filled.\\
	   You should not change this variable.
	   @see dcoords
	   @memo
	   */
	 bool filled;
	 /**
	   An array of {\tt QCOORD} defining each object to draw.\\
	   You should not change this variable.  Instead, you should
	   define a \Ref{RawPixData} variable and use this variable in a
	   constructor call.
	   @memo
	   */
	 QCOORD **dcoords;
	 /**
	   Array of integers specifying the number of {\tt QCOORD} for
	   each object.\\
	   You should not change this variable.
	   @see dcoords
	   @memo
	   */
	 int *npoints;
	 /**
	   Array of character flags specifying the typ eof object to draw.\\
	   You should not change this variable.
	   @see dcoords
	   @memo
	   */
	 char *drawtype;
	 /**
	   Class Constructor. Creates an empty {\bf PixData} object.
	   @memo
	   */
	 PixData( void );
	 /**
	   Copy Constructor.
	   @memo
	   */
	 PixData( PixData& pd );
	 /**
	   Class Constructor.
	   This is the one you should use to define a {\bf PixData}
	   object using information in the \Ref{RawPixData} variable, {\em
	   rawdat}.  {\em bscale} specified a default rescaling factor,
	   which is multiplied by the default scale of the
	   \Ref{RawPixData} to calculate the size of the resulting {\bf
	   QPixmap}.
	   @memo
	   */
	 PixData( RawPixData rawdat, float bscale=1.0 );
	 /**
	   Rescales the size of the resulting {\bf QPixmap} by a factor
	   of {\em bscale}.  Rescaling is done proportionately, with the
	   size of the pixmap and all vector-coordinates defining objects
	   multiplied accordingly.\\
	   You must recall \Ref{toPixmap}() to make your changes active.
	   @memo
	   */
	 void rescale( float bscale=1.0 );
	 /**
	   Sets the foreground and background color of the resulting {\bf
	   QPixmap}.\\
	   You must recall \Ref{toPixmap}() to make your changes active.
	   @memo
	   */
	 void setColor( const QColor& fore, const QColor& back );
	 /**
	   Converts the raw data to a {\bf QPixmap}.  This is the method
	   you call when you want to update any changes you've made via
	   \Ref{rescale}() or \Ref{setColor}().
	   @memo
	   */
	 QPixmap toPixmap( void );
	 /**
	   Assignment operator.  Duplicates the {\bf PixData} object
	   during the assignment.
	   @memo
	   */
	 PixData& operator=(PixData& pd);

	 //
   protected:
	 float width0, height0;
	 int scale0;
	 float **dcoords0;
	 QColor fg;
	 QColor bg;
	 void str2arr( char *strbuf, int *arr, int npts );
	 void str2arr( char *strbuf, float *arr, int npts );
};


#endif //_PixData_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
